@@ -59,7 +59,7 @@ class Client:
         self._max_batch_size = 400 # 400 seems optimal for our message size
         self._pool = [] # List of channels for all DROID servers
         self._pool_itr = None # Iterator for client-side load balancing
-        self._renew_pool()
+        self.renew_pool()
 
     def  __repr__(self) -> str:
         return f"""
@@ -105,7 +105,7 @@ Droid Client:
         if len(self._pool) == 0:
             raise ConnectionError("No available servers.")
 
-    def _renew_pool(self) -> None:
+    def renew_pool(self) -> None:
         """Updates pool of subclients with available servers."""
         self._remove_dead_subclients()
         pool = self._pool
@@ -138,8 +138,9 @@ Droid Client:
     async def close(self) -> None:
         """Closes the all subclient connections."""
         for subclient in self._pool:
-            await subclient["channel"].close()
-        self._remove_dead_subclients()
+            print(self._pool)
+            subclient["channel"].close()
+        assert self.connections == 0, "Failed to close all connections."
 
     async def create_bot(
             self,
@@ -148,12 +149,12 @@ Droid Client:
             bot_id: str,
             investment_amount: np.float32,
             price: np.float32,
-            margin: Optional[np.float16],
-            fraction: Optional[bool],
-            multiplier_1: Optional[np.float16],
-            multiplier_2: Optional[np.float16],
-            r: Optional[np.float32],
-            q: Optional[np.float32],
+            margin: Optional[np.float16] = None,
+            fraction: Optional[bool] = None,
+            multiplier_1: Optional[np.float16] = None,
+            multiplier_2: Optional[np.float16] = None,
+            r: Optional[np.float32] = None,
+            q: Optional[np.float32] = None,
         ) -> Dict[str, NDArray]:
         """
         Create a single bot.
@@ -179,7 +180,7 @@ Droid Client:
         """
         inputs = locals().copy()
         inputs.pop('self')
-        inputs = {k: np.array([v]) for k, v in inputs.items()}
+        inputs = {k: np.array([v]) for k, v in inputs.items() if v is not None}
         resp = await self.create_bots(**inputs)
         return {k: v[0] for k, v in resp.items() if v is not None}
     
@@ -190,12 +191,12 @@ Droid Client:
             bot_id: NDArray[str],
             investment_amount: NDArray[np.float32],
             price: NDArray[np.float32],
-            margin: Optional[NDArray[np.float16]],
-            fraction: Optional[NDArray[bool]],
-            multiplier_1: Optional[NDArray[np.float16]],
-            multiplier_2: Optional[NDArray[np.float16]],
-            r: Optional[np.float32],
-            q: Optional[np.float32],
+            margin: Optional[NDArray[np.float16]] = None,
+            fraction: Optional[NDArray[bool]] = None,
+            multiplier_1: Optional[NDArray[np.float16]] = None,
+            multiplier_2: Optional[NDArray[np.float16]] = None,
+            r: Optional[np.float32] = None,
+            q: Optional[np.float32] = None,
         ) -> Dict[str, NDArray]:
         """
         Creates a batch of bots from a dictionary of inputs.
@@ -254,8 +255,8 @@ Droid Client:
             expire_date: NDArray[np.datetime64],
             price_level_1: NDArray[np.float32],
             price_level_2: NDArray[np.float32],
-            r: Optional[np.float32],
-            q: Optional[np.float32],
+            r: Optional[np.float32] = None,
+            q: Optional[np.float32] = None,
         ) -> Dict[str, NDArray]:    
         """
         Hedge a single bot.
@@ -300,8 +301,8 @@ Droid Client:
             expire_date: NDArray[np.datetime64],
             price_level_1: NDArray[np.float32],
             price_level_2: NDArray[np.float32],
-            r: Optional[np.float32],
-            q: Optional[np.float32],
+            r: Optional[np.float32] = None,
+            q: Optional[np.float32] = None,
         ) -> Dict[str, NDArray[Any]]:
         """
         Hedges a batch of bots from a dictionary of inputs.
